﻿using Dynamicweb.Content;
using Dynamicweb.Ecommerce.LiveIntegration.Configuration;
using Dynamicweb.Ecommerce.Orders;
using Dynamicweb.Extensibility;
using Dynamicweb.Extensibility.Notifications;

namespace Dynamicweb.Ecommerce.LiveIntegration.NotificationSubscribers
{
    /// <summary>
    /// Sends the order to the ERP whe it's complete.
    /// </summary>
    /// <seealso cref="NotificationSubscriberBase" />
    [Subscribe(Ecommerce.Notifications.Ecommerce.Cart.CheckoutDoneOrderIsComplete)]
    public class CartCheckoutDoneOrderIsComplete : NotificationSubscriberBase
    {
        /// <summary>
        /// Handles the notification.
        /// </summary>
        /// <param name="notification">The notification.</param>
        /// <param name="args">The args.</param>
        public override void OnNotify(string notification, NotificationArgs args)
        {
            if (args != null)
            {
                var myArgs = (Ecommerce.Notifications.Ecommerce.Cart.CheckoutDoneOrderIsCompleteArgs)args;

                if (myArgs.Order == null || myArgs.Order.OrderLines.Count <= 0 || !EnabledAndActive() ||
                    !Global.IntegrationEnabledFor(myArgs.Order.ShopId) ||
                    !Global.EnableCartCommunication(myArgs.Order.Complete) ||
                    !IsEcomShopAllowed(myArgs.Order))
                {
                    return;
                }

                if (myArgs.Order.Complete && myArgs.Order.Currency != Common.Context.Currency)
                {
                    // This can happen when redirected back from some CheckOut handler
                    // Common.Context.Currency can be set to the default Application currency,
                    // while the completed order can be in another currency
                    Common.Context.Currency = myArgs.Order.Currency;
                }

                bool? result = OrderHandler.UpdateOrder(myArgs.Order, SubmitType.LiveOrderOrCart);

                // clear cached prices to update stock if order is completed with success
                if (result.HasValue && result.Value)
                {
                    SetProductInformation(myArgs.Order);
                }
            }
        }

        /// <summary>
        /// Determines whether ERP communication for the current shop is allowed.
        /// </summary>
        /// <param name="order">The order.</param>
        /// <returns><c>true</c> if shop is allowed; otherwise, <c>false</c>.</returns>
        private bool IsEcomShopAllowed(Order order)
        {
            bool ret = true;
            string shopId = Settings.Instance.ShopId;
            if (!string.IsNullOrEmpty(shopId))
            {
                if (Dynamicweb.Frontend.PageView.Current() != null && Dynamicweb.Frontend.PageView.Current().Area != null)
                {
                    if (Dynamicweb.Frontend.PageView.Current().Area.EcomShopId != shopId)
                    {
                        ret = false;
                    }
                }
                else
                {
                    IPageService pageService = ServiceLocator.Current.GetPageService();
                    if (pageService != null)
                    {
                        Page page = pageService.GetPage(order.CheckoutPageId);
                        if (page != null)
                        {
                            IAreaService areaService = ServiceLocator.Current.GetAreaService();
                            if (areaService != null)
                            {
                                Area area = areaService.GetArea(page.AreaId);
                                if (area != null)
                                {
                                    if (area.EcomShopId != shopId)
                                    {
                                        ret = false;
                                    }
                                    else
                                    {
                                        // add shopid for UrlHandler to find the correct url
                                        Context.Current.Items.Add("shopid", area.EcomShopId);
                                    }
                                }
                            }
                        }
                    }
                }
            }

            return ret;
        }
    }
}