﻿using Dynamicweb.Ecommerce.LiveIntegration.XmlGenerators;
using Dynamicweb.Extensibility;
using Dynamicweb.Extensibility.Notifications;
using System;
using System.Xml;

namespace Dynamicweb.Ecommerce.LiveIntegration.Notifications
{
    /// <summary>
    /// Notifications and argument classes for notification subscribers fired during generation of order XML.
    /// </summary>
    public static class Order
    {
        /// <summary>
        /// Occurs before the XML for a cart or order is generated. This enables you to change or analyze the order before it's processed.
        /// </summary>
        /// <example>
        /// <code description="Notification observer example" source="..\..\s\Dynamicweb\Integrations\Dynamicweb.Ecommerce.LiveIntegration\Head\Dynamicweb.Ecommerce.LiveIntegration.Examples\Notifications\OrderBeforeGenerateXmlSubscriber.cs" lang="CS"></code>
        /// </example>
        public const string OnBeforeGenerateOrderXml = "Dynamicweb.Ecommerce.LiveIntegration.Notifications.LiveIntegration.OnBeforeGenerateOrderXml";

        /// <summary>
        /// Occurs after the XML for a cart or order is generated. This enables you to change or analyze the XML before it's sent to the ERP.
        /// </summary>
        /// <example>
        /// <code description="Notification observer example" source="..\..\s\Dynamicweb\Integrations\Dynamicweb.Ecommerce.LiveIntegration\Head\Dynamicweb.Ecommerce.LiveIntegration.Examples\Notifications\OrderAfterGenerateXmlSubscriber.cs" lang="CS"></code>
        /// </example>
        public const string OnAfterGenerateOrderXml = "Dynamicweb.Ecommerce.LiveIntegration.Notifications.LiveIntegration.OnAfterGenerateOrderXml";

        /// <summary>
        /// Occurs before a cart or order is sent to the ERP.
        /// </summary>
        /// <example>
        /// <code description="Notification observer example" source="..\..\s\Dynamicweb\Integrations\Dynamicweb.Ecommerce.LiveIntegration\Head\Dynamicweb.Ecommerce.LiveIntegration.Examples\Notifications\OrderBeforeSendToErpSubscriber.cs" lang="CS"></code>
        /// </example>
        public const string OnBeforeSendingOrderToErp = "Dynamicweb.Ecommerce.LiveIntegration.Notifications.LiveIntegration.OnBeforeSendingOrderToErp";

        /// <summary>
        /// Occurs after a cart or order has been sent to the ERP.
        /// </summary>
        /// <example>
        /// <code description="Notification observer example" source="..\..\s\Dynamicweb\Integrations\Dynamicweb.Ecommerce.LiveIntegration\Head\Dynamicweb.Ecommerce.LiveIntegration.Examples\Notifications\OrderAfterSendToErpSubscriber.cs" lang="CS"></code>
        /// </example>
        public const string OnAfterSendingOrderToErp = "Dynamicweb.Ecommerce.LiveIntegration.Notifications.LiveIntegration.OnAfterSendingOrderToErp";

        /// <summary>
        /// Arguments class for the OnBeforeGenerateOrderXml subscriber.
        /// </summary>
        /// <seealso cref="NotificationArgs" />
        public class OnBeforeGenerateOrderXmlArgs : NotificationArgs
        {
            /// <summary>
            /// Initializes a new instance of the <see cref="OnBeforeGenerateOrderXmlArgs"/> class.
            /// </summary>
            /// <param name="order">The order for which the XML is being generated.</param>
            /// <param name="settings">An instance of OrderXmlGeneratorSettings that determines how the XML is generated.</param>
            public OnBeforeGenerateOrderXmlArgs(Orders.Order order, OrderXmlGeneratorSettings settings)
            {
                Order = order;
                GeneratorSettings = settings;
            }

            /// <summary>
            /// Gets the order for which the XML is being generated.
            /// </summary>
            /// <value>The order.</value>
            public Orders.Order Order { get; }

            /// <summary>
            /// Gets the OrderXmlGeneratorSettings that determines how the XML is generated
            /// </summary>
            /// <value>The generator settings.</value>
            public OrderXmlGeneratorSettings GeneratorSettings { get; }
        }

        /// <summary>
        /// Arguments class for the OnAfterGenerateOrderXml subscriber.
        /// </summary>
        /// <seealso cref="NotificationArgs" />
        public class OnAfterGenerateOrderXmlArgs : NotificationArgs
        {
            /// <summary>
            /// Initializes a new instance of the <see cref="OnAfterGenerateOrderXmlArgs"/> class.
            /// </summary>
            /// <param name="order">The order for which the XML has been generated.</param>
            /// <param name="settings">The settings used for generating the XML.</param>
            /// <param name="xmlDocument">The XML document that has been created. You can manipulate this document to alter the XML being sent to the ERP.</param>
            public OnAfterGenerateOrderXmlArgs(Orders.Order order, OrderXmlGeneratorSettings settings, XmlDocument xmlDocument)
            {
                Order = order;
                GeneratorSettings = settings;
                Document = xmlDocument;
            }

            /// <summary>
            /// Gets the order for which the XML has been generated..
            /// </summary>
            /// <value>The order.</value>
            public Orders.Order Order { get; }

            /// <summary>
            /// Gets the generator settings.
            /// </summary>
            /// <value>The generator settings.</value>
            public OrderXmlGeneratorSettings GeneratorSettings { get; }

            /// <summary>
            /// Gets the XML document that has been created. You can manipulate this document to alter the XML being sent to the ERP..
            /// </summary>
            /// <value>The document.</value>
            public XmlDocument Document { get; }
        }

        /// <summary>
        /// Arguments class for the OnBeforeSendingOrderToErp subscriber.
        /// When Cancel is true the order is not sent.
        /// </summary>
        /// <seealso cref="CancelableNotificationArgs" />
        public class OnBeforeSendingOrderToErpArgs : CancelableNotificationArgs
        {
            /// <summary>
            /// Initializes a new instance of the <see cref="OnBeforeSendingOrderToErpArgs"/> class.
            /// </summary>
            /// <param name="order">The order that is about to be sent.</param>
            /// <param name="createOrder">True when the order should be created in the ERP, false when it's a cart calculation.</param>
            public OnBeforeSendingOrderToErpArgs(Orders.Order order, bool createOrder)
            {
                Order = order;
                CreateOrder = createOrder;
            }

            /// <summary>
            /// Gets the order that is about to be sent.
            /// </summary>
            /// <value>The order.</value>
            public Orders.Order Order { get; }

            /// <summary>
            /// Gets a value that indicates whether the order should be created in the ERP or just calculated as a cart.
            /// </summary>
            /// <value>True when the order should be created in the ERP, false when it's a cart calculation.</value>
            public bool CreateOrder { get; }
        }

        /// <summary>
        /// Arguments class for the OnAfterSendingOrderToErp subscriber.
        /// </summary>
        /// <seealso cref="NotificationArgs" />
        public class OnAfterSendingOrderToErpArgs : NotificationArgs
        {
            /// <summary>
            /// Initializes a new instance of the <see cref="OnAfterSendingOrderToErpArgs"/> class.
            /// </summary>
            /// <param name="order">The order that has been sent to the ERP.</param>
            /// <param name="createOrder">True when the order should be created in the ERP, false when it's a cart calculation.</param>
            /// <param name="responseDocument">The XML document with the ERP response.</param>
            /// <param name="error">The error that occurred, of any.</param>
            public OnAfterSendingOrderToErpArgs(Orders.Order order, bool createOrder, XmlDocument responseDocument, Exception error)
            {
                Order = order;
                CreateOrder = createOrder;
                ResponseDocument = responseDocument;
                Error = error;
            }

            /// <summary>
            /// Gets the order that has been sent to the ERP.
            /// </summary>
            /// <value>The order.</value>
            public Orders.Order Order { get; }

            /// <summary>
            /// Gets a value that indicates whether the order should be created in the ERP or just calculated as a cart.
            /// </summary>
            /// <value>True when the order should be created in the ERP, false when it's a cart calculation.</value>
            public bool CreateOrder { get; }

            /// <summary>
            /// Gets the XML response document.
            /// </summary>
            /// <value>The response document.</value>
            public XmlDocument ResponseDocument { get; }

            /// <summary>
            /// Gets the error that occurred, if any.
            /// </summary>
            /// <value>The error.</value>
            public Exception Error { get; }
        }
    }
}